;Sibelius Access version 1 beta 2
;JAWS scripts for Sibelius 5.2
;For JAWS 7.1.500 and above.
;by Daniel Craig Rugman (Dan Rugman)
;Last revised 14/10/2008
;Sibelius Access is subject to the terms and conditions of the GNU General Public License.
;The full text of the license can be found in the file license.txt distributed with these scripts.

;includes from default
Include "HjGlobal.jsh" ; default HJ global variables
Include "hjconst.jsh" ; default HJ constants
Include "HjHelp.jsh" ; Help Topic Constants
Include "common.jsm" ; message file
include "Sibelius5Constants.JSH"
include "Sibelius5Globals.JSH"
include "sibelius5.JSM"
Include "Sibelius5WinFunc.jsm"

Void Function BugReport ()
var
handle hTaskBar,
string sLine,
string sPath,
string sSec
let sPath = gsDataStore
WipeIniFile(sPath)
let sSec = "General"
let sLine = FormatString (cmsgGetSysTime, SysGetTime ()) +space+
FormatString (cmsgGetSysDate, SysGetDate ())
IniWriteString (sSec, "Time", sLine, sPath)
IniWriteString (sSec, "User", GetJAWSUserName (), sPath)
IniWriteInteger (sSec, "JAWS version", GetJFWVersion (), sPath)
IniWriteString (sSec, "Sibelius version", GetVersionInfoString (GetAppFilePath (), cmsg283_L), sPath)
IniWriteString (sSec, "Input language", GetInputLanguage(), sPath)
let sSec = "Windows"
let sLine = IntToString(ScreenGetWidth()) + "x" + IntToString(ScreenGetHeight())
IniWriteString (sSec, "Screen", 		sLine, sPath)
let hTaskBar = FindTopLevelWindow (cwcShellTray, cscNull)
IniWriteString (sSec, "TaskBar", GetWindowData (hTaskBar), sPath)
IniWriteString (sSec, "MDIFrame window", GetWindowData (GetSibeliusFrameWindow()), sPath)
IniWriteString (sSec, Name_Properties, GetWindowData (ghPropertiesWindow), sPath)
IniWriteString (sSec, Name_Keypad, GetWindowData (ghKeypadWindow), sPath)
IniWriteString (sSec, Name_Playback, GetWindowData (ghPlaybackWindow), sPath)
IniWriteString (sSec, Name_Navigator, GetWindowData (ghNavigatorWindow), sPath)
IniWriteString (sSec, Name_Mixer, GetWindowData (ghMixerWindow), sPath)
IniWriteString (sSec, Name_Parts, GetWindowData (ghPartsWindow), sPath)
IniWriteString (sSec, Name_Video, GetWindowData (ghVideoWindow), sPath)
Run (MsgQuote+sPath+MsgQuote)
EndFunction

String Function GetWindowData (handle hWnd)
var
int iLeft, int iTop, int iRight, int iBottum,
string State,
string msg
if IsWindowVisible (hWnd) then
let State= cmsg343_L ;visible
else
let State= cmsg344_L ;not visible
endif
let msg = FormatString( MsgWindowDataForDebug, State,
FormatWindowCoordinates(hWnd))
return msg
EndFunction


String Function FormatWindowCoordinates (handle hWnd)
return FormatString (MsgWindowCoordinates,
IntToString(GetWindowLeft(hWnd)),
IntToString(GetWindowTop(hWnd)),
IntToString(GetWindowRight(hWnd)),
IntToString(GetWindowBottom(hWnd)))
EndFunction


Void Function WipeIniFile (string Path)
var
string List,
string Section,
int Index
if !FileExists (Path) then
return
endif
let List = IniReadSectionNames (Path)
let Index = 1
while Index <= StringSegmentCount(List, delimiter)
let Section = StringSegment (List, delimiter, Index)
IniRemoveSection (Section, Path)
let Index = Index+1
endwhile
EndFunction


Function WipeTrace ()
;wipe the ini file if there is one.
if FileExists (gsToolWindowTrace) then
IniRemoveSection ("log", gsToolWindowTrace, True)
endif
EndFunction


Function WriteToTrace (string sLine)
;Adds a line to the trace file.
;The line will appear as a string value to an ini key.
;The name of the key will be the next integer available within the section called "log".
var
string sAllKeys,
int Index,
string sLastKey
if !FileExists (gsToolWindowTrace) then
SayString("cannot find trace file")
return
endif
;The names of the keys are numbers in order 1, 2, 3 ...
;Get all the nnames of all keys.
let sAllKeys = IniReadSectionKeys ("log", gsToolWindowTrace)
;Get the name of the last key.
let Index = StringSegmentCount (sAllKeys, delimiter)
if Index == 1 then
let sLastKey = "1"
else
let sLastKey = StringSegment (sAllKeys, delimiter, Index)
;Get the next number in the sequence.
let sLastKey = IntToString (StringToInt(sLastKey) + 1)
endif
IniWriteString ("Log", sLastKey, sLine, gsToolWindowTrace)
EndFunction


Function WriteErrorToTrace (string sLine)
IniWriteString ("Log", MsgTraceErrorNotice, sLine, gsToolWindowTrace)
EndFunction

Function TimeStampTrace ()
var
string sLine
let sLine = FormatString (cmsgGetSysTime, SysGetTime ()) +space+
FormatString (cmsgGetSysDate, SysGetDate ())
IniWriteString ("Log", "Time", sLine, gsToolWindowTrace)
EndFunction


String Function GetErrorFromTrace ()
return IniReadString ("Log", MsgTraceErrorNotice, "zzz", gsToolWindowTrace)
EndFunction


Void Function GetExpectedCoordinatesOfToolWindows ()
;Fills the globals with the expected coordinates of the tool windows.
;These have to be calculated to work around different screen resolutions.
var
int ScreenWidth
let ScreenWidth = ScreenGetWidth ()
;The right edge of the properties window is flush with the left edge of the screen.
let giExpectedPropertiesLeft = ScreenWidth - ciPropertiesWidth
;The right edge of the keypad window is flush with the left edge of the properties window.
let giExpectedKeypadLeft = giExpectedPropertiesLeft - ciKeypadWidth - ciToolWindowSep
;The right edge of the playback window is flush with the left edge of the keypad window.
let giExpectedPlaybackLeft = giExpectedKeypadLeft - ciPlaybackWidth - ciToolWindowSep
EndFunction

Void Function ToggleToolWindow (string Name)
var
handle hWnd,
int IsWindowOpen,
int WindowShouldOpen,
string msg
let hWnd = GetToolWindow( Name)
;The open/closed state of the window must be retrieved first for comparison.
let IsWindowOpen = IsWindowVisible (hWnd)
if IsWindowOpen then
if CloseToolWindow (hWnd, Name) then
SayFormattedMessage( OT_NO_DISABLE, FormatString( MsgToolWindowClosed, Name))
return True
endif
elif !IsWindowOpen then
if OpenToolWindow (hWnd, Name) then
SayFormattedMessage( OT_NO_DISABLE, FormatString( MsgToolWindowOpen, Name))
return
endif
endif
SayFormattedMessage (OT_USER_BUFFER, ErrorMessageForToolWindowOpen (Name))
EndFunction

Int Function OpenToolWindow (handle hWnd, string Name)
;No action is necessary if the window is already in the correct state
if IsWindowVisible (hWnd) then
return True
endif
;Pass on the keystroke for the tool window
;The keystroke in the JKM file must match the keystroke set in the Sibelius preferences dialog.
TypeKey (GetToolWindowKeyStroke (Name))
;Wait before checking the status of the window
;The delay is contained in the checking function. This avoids the scheduling problem.
if IsToolWindowOpen (hWnd, 1) then
return True
endif
;The window hasn't responded. Wait and check again.
if IsToolWindowOpen (hWnd, 10) then
return True
endif
;The keystroke failed to open the window.
return False
EndFunction

Int Function CloseToolWindow (handle hWnd, string Name)
;No action is necessary if the window is already in the correct state
if !IsWindowVisible (hWnd) then
return True
endif
;Pass on the keystroke for the tool window
;The keystroke in the JKM file must match the keystroke set in the Sibelius preferences dialog.
TypeKey (GetToolWindowKeyStroke (Name))
;Wait before checking the status of the window
;The delay is contained in the checking function. This avoids the scheduling problem.
if !IsToolWindowOpen (hWnd, 1) then
return True
endif
;The window hasn't responded. Wait and check again.
if !IsToolWindowOpen (hWnd, 10) then
return True
endif
;The keystroke failed to open the window.
return False
EndFunction

Int Function IsToolWindowOpen (handle hWnd, int Delay)
;This just checks the visibility of the tool window.
;The delay is essential and is included within the function to avoid scheduling errors which occur with multiple delays and conditionals.
Delay (Delay)
if IsWindowVisible (hWnd) then
return True
else
return False
endif
EndFunction

String Function ErrorMessageForToolWindowOpen (string Name)
;This generates the error message to be displayed when a tool window fails to open or close.
;It is written as a function because this string may be added to the debug log when running FixToolWindows
var
string Msg,
string Keystroke
let Keystroke = GetToolWindowKeyStroke (Name)
let Msg = FormatString (MsgToolWindowKeystrokeAlert, Keystroke, Name)
if IsWindowVisible (GetToolWindow( Name)) then
;The window is open so it failed to close.
return FormatString (MsgToolWindowCloseFailed, Name, Keystroke) +"\r\n"+ Msg
else
;The window is closed so it failed to open.
return FormatString (MsgToolWindowOpenFailed, Name, Keystroke) +"\r\n"+ Msg
endif
EndFunction

String Function GetToolWindowKeyStroke (string Name)
return GetScriptKeyName ("OpenAndClose" + Name)
EndFunction


HANDLE Function GetToolWindow (string Name)
if Name == Name_Properties then
GetPropertiesWindow()
return ghPropertiesWindow
elif Name == Name_Keypad then
GetKeypadWindow()
return ghKeypadWindow
elif Name == Name_Playback then
GetPlaybackWindow()
return ghPlaybackWindow
elif Name == Name_Mixer then
GetMixerWindow()
return ghMixerWindow
elif Name == Name_Parts then
GetPartsWindow()
return ghPartsWindow
elif Name == Name_Navigator then
GetNavigatorWindow()
return ghNavigatorWindow
elif Name == Name_Video then
GetVideoWindow()
return ghVideoWindow
endif
EndFunction


Int Function GetPropertiesWindow ()
var
handle hWnd,
int Continue
if GetWindowClass (ghPropertiesWindow) == cWcToolWindow &&
GetWindowClass (GetFirstChild (ghPropertiesWindow)) == cWcPropertiesPain then
return True
endif
; find the properties window
let hWnd = FindWindow (0, cwcToolWindow)
let Continue = TRUE
While Continue == TRUE
if GetWindowClass (hWnd) == cWcToolWindow &&
GetWindowClass (GetFirstChild (hWnd)) == cWcPropertiesPain then
let Continue = FALSE
elif hWnd == 0 then
let Continue = FALSE
else
let hWnd = GetNextWindow (hWnd)
EndIf
EndWhile
if GetWindowClass (hWnd) == cWcToolWindow &&
GetWindowClass (GetFirstChild (hWnd)) == cWcPropertiesPain then
let ghPropertiesWindow = hWnd
return True
else
return False
endif
EndFunction


Int Function GetKeypadWindow ()
if GetWindowName (ghKeypadWindow) == Name_Keypad &&
GetWindowClass (ghKeypadWindow) == cwcToolWindow then
return True
endif
let ghKeypadWindow = FindWindow (0, cWcToolWindow, Name_Keypad)
if GetWindowName (ghKeypadWindow) == Name_Keypad &&
GetWindowClass (ghKeypadWindow) == cwcToolWindow then
return True
else
return False
endif
EndFunction


Int Function GetPlaybackWindow ()
if GetWindowName (ghPlaybackWindow) == Name_Playback &&
GetWindowClass (ghPlaybackWindow) == cwcToolWindow then
return True
endif
let ghPlaybackWindow = FindWindow (0, cWcToolWindow, Name_Playback)
if GetWindowName (ghPlaybackWindow) == Name_Playback &&
GetWindowClass (ghPlaybackWindow) == cwcToolWindow then
return True
else
return False
endif
EndFunction


Int Function GetPartsWindow ()
if GetWindowName (ghPartsWindow) == Name_Parts &&
GetWindowClass (ghPartsWindow) == cwcToolWindow then
return True
endif
let ghPartsWindow = FindWindow (0, cWcToolWindow, Name_Parts)
if GetWindowName (ghPartsWindow) == Name_Parts &&
GetWindowClass (ghPartsWindow) == cwcToolWindow then
return True
else
return False
endif
EndFunction


Int Function GetNavigatorWindow ()
if GetWindowName (ghNavigatorWindow) == Name_Navigator &&
GetWindowClass (ghNavigatorWindow) == cwcToolWindow then
return True
endif
let ghNavigatorWindow =  FindWindow (0, cWcToolWindow, Name_Navigator)
if GetWindowName (ghNavigatorWindow) == Name_Navigator &&
GetWindowClass (ghNavigatorWindow) == cwcToolWindow then
return True
else
return False
endif
EndFunction


Int Function GetMixerWindow ()
var
handle hWnd,
handle NullWin,
int Continue

if GetWindowName (ghMixerWindow) == Name_Mixer &&
GetWindowClass (ghMixerWindow) == cwcMixer then
return True
endif

;The class of the mixer window is not consistent, although it always begins with "WindowsForms10.Window.20008.app.0",
;This means that the builtins like FindTopLevelWindow will not work
;This is because they do compare the entire length of both strings, unlike the standard equality operator. Silly!
;Start by getting the first top level window in the OSM
let hWnd = GetFirstWindow (FindTopLevelWindow (cwcShellTray, cscNull))

;Search the top level windows for one called Mixer
let Continue = TRUE
While Continue == TRUE
if GetWindowClass (hWnd) == cwcMixer
&& GetWindowName (hWnd) == Name_Mixer then
let ghMixerWindow = hWnd
let Continue = FALSE
elif hWnd == NullWin then
let Continue = FALSE
else
let hWnd = GetNextWindow (hWnd)
EndIf
EndWhile

if hWnd == NullWin then
return False
else
return True
endif

EndFunction


Int Function GetVideoWindow ()
if GetWindowName (ghVideoWindow) == Name_Video &&
GetWindowClass (ghVideoWindow) == cwcToolWindow then
return True
endif
let ghVideoWindow = FindWindow (0, cWcToolWindow, Name_Video)
if GetWindowName (ghVideoWindow) == Name_Video &&
GetWindowClass (ghVideoWindow) == cwcToolWindow then
return True
else
return False
endif
EndFunction

String Function AreToolWindowsMissing ()
var
string Debug
if !GetPropertiesWindow() then
let Debug = Debug +newline+ Name_Properties
endif
if !GetKeypadWindow() then
let Debug = Debug +newline+ Name_Keypad
endif
if!GetPlaybackWindow() then
let Debug = Debug +newline+ Name_Playback
endif
if !GetNavigatorWindow() then
let Debug = Debug +newline+ Name_Navigator
endif
if !GetMixerWindow() then
let Debug = Debug +newline+ Name_Mixer
endif
if !GetPartsWindow() then
let Debug = Debug +newline+ Name_Parts
endif
if !GetVideoWindow() then
let Debug = Debug +newline+ Name_Video
endif
return Debug
EndFunction

Void Function GoToScore ()
var
handle hWnd
GetAllToolWindows()
let hWnd = GetTopLevelWindow (GetFocus())
if hWnd != ghKeypadWindow &&
hWnd != ghPropertiesWindow &&
hWnd != ghPlaybackWindow &&
hWnd != ghMixerWindow &&
hWnd != ghPartsWindow &&
hWnd != ghVideoWindow then
SayFormattedMessage (OT_NO_DISABLE, MsgScoreFocusAbort)
return
endif
let hWnd = GetScoreWindow()
if !hWnd then
SayFormattedMessage (OT_NO_DISABLE, MsgNoScoresOpen)
return
endif
SetFocus (hWnd)
EndFunction

Function GiveToolWindowFocus (int Direction)
var
handle hWnd,
handle hScore,
string ListOfHandles,
int Index
let hWnd = GetFocus()
let hScore = GetScoreWindow()
AreToolWindowsMissing()
if !hScore then
SayFormattedMessage (OT_NO_DISABLE, MsgGiveToolWindowFocusNoScore)
return
endif
if hWnd != hScore then
let hWnd = GetTopLevelWindow(hWnd)
endif
if hWnd != hScore &&
hWnd != ghKeypadWindow &&
hWnd != ghPropertiesWindow &&
hWnd != ghPlaybackWindow &&
hWnd != ghMixerWindow &&
hWnd != ghPartsWindow then
SayFormattedMessage (OT_NO_DISABLE, MsgGiveToolWindowFocusWrongFocus)
return
endif
let ListOfHandles =
AddToolWindowToList (ghPartsWindow)+
AddToolWindowToList (ghMixerWindow)+
AddToolWindowToList (ghPropertiesWindow)+
AddToolWindowToList (ghPlaybackWindow) +
AddToolWindowToList (ghKeypadWindow)+
IntToString(hScore)
let Index = StringSegmentIndex (ListOfHandles, delimiter, IntToString(hWnd))
if Direction == UnitMove_Next then
if Index == StringSegmentCount (ListOfHandles, delimiter) then
let Index = 1
else
let Index = Index + 1
endif
else
if Index == 1 then
let Index = StringSegmentCount (ListOfHandles, delimiter)
else
let Index = Index - 1
endif
endif
let hWnd = StringToHandle( StringSegment (ListOfHandles, delimiter, Index))
let gMovingToToolWindow = True
SetFocus (hWnd)
;Announcements come from functions triggered by HandleCustomeWindows
EndFunction

String Function AddToolWindowToList (handle hWnd)
if IsWindowVisible (hWnd) then
return IntToString (hWnd) + delimiter
else
return cscNull
endif
EndFunction


String Function GetToolWindowName ()
var
handle hWnd,
string sName,
string Msg
let gMovingToToolWindow = False
let hWnd = GetTopLevelWindow (GetFocus ())
if hWnd == ghPropertiesWindow then
SayFormattedMessage (OT_NO_DISABLE, Name_Properties)
SayFormattedMessage (OT_NO_DISABLE, ListOpenPropertiesPains())
elif hWnd == ghMixerWindow then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowName, GetWindowName(hWnd)))
SayMixerSizeAlert()
else
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowName, GetWindowName(hWnd)))
endif
EndFunction

String Function GetKeyPressesForMove (handle hWnd, int ByRef iUpArrows, int ByRef iDownArrows, int ByRef iLeftArrows, int ByRef iRightArrows)
;Returns by reference the number of times that the arrow keys need to be pressed to move the window back to its proper position.
var
int iLeft, int iTop
;Get the difference between the expected coordinates and actual coordinates of the window.
let iTop = 0- GetWindowTop(hWnd)
if hWnd == ghKeypadWindow then
let iLeft = giExpectedKeypadLeft - GetWindowLeft(hWnd)
elif hWnd == ghPropertiesWindow then
let iLeft = giExpectedPropertiesLeft - GetWindowLeft(hWnd)
elif hWnd == ghPlaybackWindow then
let iLeft = giExpectedPlaybackLeft - GetWindowLeft(hWnd)
endif
;Determine the number of keystrokes needed to move the window back.
let iUpArrows = 0
let iDownArrows = 0
let iLeftArrows = 0
let iRightArrows = 0
if iLeft > 0 then
let iRightArrows = iLeft / 9
elif iLeft < 0 then
let iLeftArrows = iLeft / -9
endif
if iTop > 0 then
let iDownArrows = iTop / 9
elif iTop < 0 then
let iUpArrows = iTop / -9
endif
;the values are returned by reference.
EndFunction

String Function GetMessageForMoveQuery ()
var
handle hWnd,
int iDown, int iUp, int iLeft, int iRight,
string sName,
string Msg,
string MsgUp, string MsgDown, string MsgLeft, string MsgRight
let gMovingToToolWindow = False
let hWnd = GetTopLevelWindow (GetFocus ())
GetKeyPressesForMove (hWnd, iUp, iDown, iLeft, iRight)
if !iUp && !iDown && !iLeft && !iRight then
;The window does not need to be moved so just announce the window name
SayFormattedMessage (OT_NO_DISABLE, MsgToolWindowCorrectPosition)
return
endif
if UserBufferIsActive() then
UserBufferDiactivate()
endif
let Msg = MsgToolWindowMoveAlert1
if iUp then
let Msg = Msg +newline+ FormatString (MsgToolWindowMoveAlert2, cksUpArrow, IntToString(iUp))
endif
if iDown then
let Msg = Msg +newline+ FormatString (MsgToolWindowMoveAlert2, cksDownArrow, IntToString(iDown))
endif
if iLeft then
let Msg = Msg +newline+ FormatString (MsgToolWindowMoveAlert2, cksLeftArrow, IntToString(iLeft))
endif
if iRight then
let Msg = Msg +newline+ FormatString (MsgToolWindowMoveAlert2, cksRightArrow, IntToString(iRight))
endif
let Msg = Msg + MsgToolWindowMoveAlert3
SayFormattedMessage (OT_USER_BUFFER, Msg)
EndFunction


Int Function GetAllToolWindows ()
if !GetPropertiesWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Properties))
return False
elif !GetKeypadWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Keypad))
return False
elif !GetPlaybackWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Playback))
return False
elif !GetMixerWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Mixer))
return False
elif !GetPartsWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Parts))
return False
elif !GetNavigatorWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Navigator))
return False
elif !GetVideoWindow() then
SayFormattedMessage (OT_No_Disable, FormatString (MsgToolWndNotFound, Name_Video))
return False
endif
return True
EndFunction

Int Function CheckToolWindowStates ()
if !IsWindowVisible( ghPropertiesWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndClosedAlert, Name_Properties))
return False
elif !IsWindowVisible( ghKeypadWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndClosedAlert, Name_Keypad))
return False
elif !IsWindowVisible( ghPlaybackWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndClosedAlert, Name_Playback))
return False
elif IsWindowVisible( ghNavigatorWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndOpenAlert, Name_Navigator))
return False
elif IsWindowVisible( ghMixerWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndOpenAlert, Name_Mixer))
return False
elif IsWindowVisible( ghPartsWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndOpenAlert, Name_Parts))
return False
elif IsWindowVisible( ghVideoWindow) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWndOpenAlert, Name_Video))
return False
endif
return True
EndFunction

Int Function OverlappingToolWindows ()
var
int TaskTop,
int KPL, int KPR, int KPT, int KPB,
int PrL, int PrR, int PrT, int PrB,
int PlL, int PlR, int PlT, int PlB,
int Vl, int VR, int VT, int VB
let TaskTop = GetWindowTop(FindTopLevelWindow (cwcShellTray, cscNull))
GetWindowRect (ghKeypadWindow, KPL, KPR, KPT, KPB)
GetWindowRect (ghPropertiesWindow, PrL, PrR, PrT, PrB)
GetWindowRect (ghPlaybackWindow, PlL, PlR, PlT, PlB)
;check for task bar collisions
if KPB > TaskTop then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgTaskBarCollision, Name_Keypad))
return True
elif PrB > TaskTop then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgTaskBarCollision, Name_Properties))
return True
elif PlB > TaskTop then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgTaskBarCollision, Name_Playback))
return True
endif
; the Keypad, properties and playback must not be overlapping
if OverlappingRects (KPL, KPR, KPT, KPB, PrL, PrR, PrT, PrB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Keypad, Name_Properties))
return True
endif
if OverlappingRects (KPL, KPR, KPT, KPB, PlL, PlR, PlT, PlB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Keypad, Name_Playback))
return True
endif
if OverlappingRects (PrL, PrR, PrT, PrB, PlL, PlR, PlT, PlB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Properties, Name_Playback))
return True
endif
;The video window may need to be open
if IsWindowVisible (ghVideoWindow) then
GetWindowRect (ghVideoWindow, VL, VR, VT, VB)
if OverlappingRects (KPL, KPR, KPT, KPB, VL, VR, VT, VB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Video, Name_Keypad))
return True
endif
if OverlappingRects (PrL, PrR, PrT, PrB, VL, VR, VT, VB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Video, Name_Properties))
return True
endif
if OverlappingRects (PlL, PlR, PlT, PlB, VL, VR, VT, VB) then
SayFormattedMessage (OT_NO_DISABLE, FormatString (MsgToolWindowCollision, Name_Video, Name_Playback))
return True
endif
endif
EndFunction

Int Function OverlappingRects (int Left1, int Right1, int Top1, int Bottom1, int Left2, int Right2, int Top2, int Bottom2)
if Right1 < Left2 ||
Right2 < Left1 ||
Bottom1 < Top2 ||
Bottom2 < Top1 then
return False
else
return True
endif
EndFunction


Int Function CheckToolWindows ()
var
handle hWnd
if !GetAllToolWindows() then
return False
elif !CheckToolWindowStates() then
return False
elif OverlappingToolWindows() then
return False
endif
;All tool windows passed.
;Store the edges of the keypad window globaly for the keypad functions.
let hwnd = GetFirstChild (ghKeypadWindow)
let giKeypadLeftEdge = GetWindowLeft (hwnd)
let giKeypadTopEdge = GetWindowTop (hwnd)
return true
EndFunction


Int Function IsToolWindowPositioned (handle hWnd, int iExpectedCol, int iExpectedRow)
var
int iLeft,
int iTop,
string sName
let iLeft = GetWindowLeft( hWnd)
let iTop = GetWindowTop( hWnd)
;tollerate a deviation of 5 pixils in each direction
if iLeft > iExpectedCol -5 &&
iLeft < iExpectedCol +5 &&
iTop > iExpectedRow -5 &&
iTop < iExpectedRow +5 then
return True
else
return False
endif
EndFunction


Int Function MaximizeMainWindow ()
var
handle hWnd
let hWnd = GetSibeliusFrameWindow ()
if GetWindowLeft(hWnd) < 0 &&
GetWindowRight(hWnd) > ScreenGetWidth() &&
GetWindowTop(hWnd) < 0 &&
GetWindowBottom(hWnd) > GetWindowTop(FindTopLevelWindow (cwcShellTray, cscNull)) then
return True
endif
PostMessage (hwnd, WM_SYSCOMMAND, SC_MAXIMIZE, 0)
Delay(5)
if GetWindowLeft(hWnd) < 0 &&
GetWindowRight(hWnd) > ScreenGetWidth() &&
GetWindowTop(hWnd) < 0 &&
GetWindowBottom(hWnd) > GetWindowTop(FindTopLevelWindow (cwcShellTray, cscNull)) then
return True
else
return False
endif
EndFunction




Void Function FixToolWindows ()
var
handle hWnd,
handle hScore,
string Debug,
string sFailures,
string msg
WipeTrace()
TimeStampTrace ()
;Do not run this function unless a score is open
let hWnd = GetFocus()
let hScore = GetScoreWindow()
;Check that all tool windows can be found.  This should not go wrong but is here just for safety..
let Debug = AreToolWindowsMissing()
if Debug then
let Debug = FormatString (MsgToolWindowFailHandle, Debug)
WriteToTrace (Debug)
SayFormattedMessage (OT_USER_BUFFER, Debug)
return False
else
WriteToTrace (MsgToolWindowsFound)
endif
if !hScore then
SayFormattedMessage (OT_NO_DISABLE, MsgFixToolWindowsNoScore)
return
elif hWnd != hScore &&
hWnd != ghKeypadWindow &&
hWnd != ghPropertiesWindow &&
hWnd != ghPlaybackWindow then
SayFormattedMessage (OT_NO_DISABLE, MsgFixToolWindowsWrongFocus)
return
endif
; Just in case
Refresh (1)
if !MaximizeMainWindow() then
;SayFormattedMessage (OT_USER_BUFFER, MsgMainWindowMaximizeAlert)
SayFormattedMessage (OT_NO_DISABLE, MsgMainWindowMaximizeAlert)
;return
endif
;Set the global which HandleCustomeWindows needs
let giFixingToolWindows = True
;The trace lines are written from within the functions which follow.
if giAutomaticallyToggleToolWindows then
if !OpenToolWindowWhenFixing (ghPropertiesWindow, Name_Properties) ||
!OpenToolWindowWhenFixing (ghKeypadWindow, Name_Keypad) ||
!OpenToolWindowWhenFixing (ghPlaybackWindow, Name_Playback)  ||
!CloseToolWindowWhenFixing (ghMixerWindow, Name_Mixer) ||
!CloseToolWindowWhenFixing (ghNavigatorWindow, Name_Navigator) ||
!CloseToolWindowWhenFixing (ghPartsWindow, Name_Parts) ||
!CloseToolWindowWhenFixing (ghVideoWindow, Name_Video) then
let giFixingToolWindows = False
ShowFixToolWindowError()
return
endif
else
if CheckToolWindowStates () then
WriteToTrace (MsgToolWndNoToggleNeeded)
else
WriteToTrace (MsgToolWndUserMustToggle)
let giFixingToolWindows = False
return
endif
endif
if !IsToolWindowPositioned (ghPropertiesWindow, giExpectedPropertiesLeft, 0) ||
!IsToolWindowPositioned (ghKeypadWindow, giExpectedKeypadLeft, 0) ||
!IsToolWindowPositioned (ghPlaybackWindow, giExpectedPlaybackLeft, 0) then
;At least one window needs to be moved. This slows down the function.
SayFormattedMessage(OT_NO_DISABLE, MsgPleaseWaite)
endif
if MoveToolWindow (ghPropertiesWindow, Name_Properties, giExpectedPropertiesLeft, 0) &&
MoveToolWindow (ghKeypadWindow, Name_Keypad, giExpectedKeypadLeft, 0) &&
MoveToolWindow (ghPlaybackWindow, Name_Playback, giExpectedPlaybackLeft, 0) then
;All operations are complete so report success.
WriteToTrace (MsgToolWindowsFixed_L)	
SayFormattedMessage (OT_NO_DISABLE, MsgToolWindowsFixed_L, MsgToolWindowsFixed_S)
else
ShowFixToolWindowError()
endif
let giFixingToolWindows = False
EndFunction


Function ShowFixToolWindowError ()
var
string msg
let msg = GetErrorFromTrace()
if msg == "zzz" then
;The error message could not be found so read the default message.
SayFormattedMessage (OT_USER_BUFFER, MsgToolWindowFixFailure)
else
SayFormattedMessage (OT_USER_BUFFER, msg)
endif
EndFunction


Int Function OpenToolWindowWhenFixing (handle hWnd, string Name)
if OpenToolWindow (hWnd, Name) then
WriteToTrace (FormatString( MsgToolWindowOpen, Name))
return True
else
WriteErrorToTrace (ErrorMessageForToolWindowOpen (Name))
return False
endif
EndFunction

Int Function CloseToolWindowWhenFixing (handle hWnd, string Name)
if CloseToolWindow (hWnd, Name) then
WriteToTrace (FormatString( MsgToolWindowClosed, Name))
return True
else
WriteErrorToTrace (ErrorMessageForToolWindowOpen (Name))
return False
endif
EndFunction

Int Function MoveToolWindow (handle hWnd, string sName, int iNewLeftEdge, int iNewTopEdge)
var
int iLeftEdge, int iTopEdge,
int iUpArrows, int iDownArrows, int iLeftArrows, int iRightArrows
if IsToolWindowPositioned (hWnd, iNewLeftEdge, iNewTopEdge) then
WriteToTrace (FormatString (MsgToolWindowNoMoveRequired, sName))
return True
else
;Add a notification to the trace file that the window needs to be moved.
;Include the coordinates of the window for error checking.
let iLeftEdge = GetWindowLeft( hWnd)
let iTopEdge = GetWindowTop (hWnd)
WriteToTrace (FormatString (MsgToolWindowMoveRequired, sName, IntToString(iLeftEdge), IntToString(iTopEdge), IntToString(iNewLeftEdge), IntToString(iNewTopEdge)))
endif
SetFocus(hWnd)
Delay(2)
if hWnd != GetFocus() then
WriteErrorToTrace (FormatString (MsgToolWindowFocusFailed, sName))
return False
else
WriteToTrace (FormatString (MsgToolWindowFocusSuccess, sName))
endif
;The tool windows can become positioned so that the top left corner is off the screen. If this happens then keystrokes must be used to move it.
if GetKeystrokesForMoveWhileFixing (iUpArrows, iDownArrows, iLeftArrows, iRightArrows, hWnd) then
MoveToolWindowWithKeystrokes (iUpArrows, iDownArrows, iLeftArrows, iRightArrows, hWnd)
Delay(5) ; The window may need time to move and stabalise
	let iLeftEdge = GetWindowLeft( hWnd)
	let iTopEdge = GetWindowTop (hWnd)
	if GetKeystrokesForMoveWhileFixing (iUpArrows, iDownArrows, iLeftArrows, iRightArrows, hWnd) then
;The window is still off screen so quit.
	WriteErrorToTrace( FormatString (MsgToolWindowMoveWithKeystrokesFailed, sName, IntToString(iLeftEdge), IntToString(iTopEdge)))
	return False
else
	WriteToTrace( FormatString (MsgToolWindowMoveWithKeystrokesSuccess, sName, IntToString(iLeftEdge), IntToString(iTopEdge)))
	endif
endif
DragItemWithMouse (iLeftEdge+5, iTopEdge+10, iNewLeftEdge +5, iNewTopEdge + 10)
Delay(5)
if !IsToolWindowPositioned (hWnd, iNewLeftEdge, iNewTopEdge) then
WriteErrorToTrace (FormatString (MsgToolWindowMoveFailed, sName))
return False
else
WriteToTrace (FormatString (MsgToolWindowMoveSuccess))
return True
endif
EndFunction

Int Function GetKeystrokesForMoveWhileFixing (int ByRef iUpArrows, int ByRef iDownArrows, int ByRef iLeftArrows, int ByRef iRightArrows, handle hWnd)
;Determines if the top left corner of the tool window passed is outside the screen boundary.
;If it is then the number of times that each arrow key must be used to move it back is passed by reference.
;The function also returns true if the window must be moved by using keystrokes.
var
int iLeftEdge, int iTopEdge,
int iTaskBarTop,
int iScreenRight
;Get the number of times the arrow keys must be pressed for the move.
let iLeftEdge = GetWindowLeft (hWnd) 
let iTopEdge = GetWindowTop(hWnd)
let iTaskBarTop = GetWindowTop(FindTopLevelWindow (cwcShellTray, cscNull)) - 20
let iScreenRight = ScreenGetWidth() - 20
if iTopEdge > iTaskBarTop then
let iUpArrows = (iTopEdge - iTaskBarTop) / 9
endif
if iTopEdge < -5 then
let iDownArrows = iTopEdge / -9
endif
if iLeftEdge > iScreenRight then
let iLeftArrows = (iLeftEdge - iScreenRight) / 9
endif
if iLeftEdge < 0 then
let iRightArrows = iLeftEdge / -9
endif
if iUpArrows || iDownArrows || iRightArrows || iLeftArrows then
return True
else
return False
endif
EndFunction

Void Function MoveToolWindowWithKeystrokes (int iUp, int iDown, int iLeft, int iRight, handle hWnd)
;Use post send to simulate choosing move from the command menu.
PostMessage (hWnd, WM_SYSCOMMAND, SC_MOVE, 0)
Pause()
if iUp then
PassKeyPressesForMove (cksUpArrow, iUp)
endif
if iDown then
PassKeyPressesForMove (cksDownArrow, iDown)
endif
if iLeft then
PassKeyPressesForMove (cksLeftArrow, iLeft)
endif
if iRight then
PassKeyPressesForMove (cksRightArrow, iRight)
endif
TypeKey("Enter")
EndFunction

Function PassKeyPressesForMove (string sKey, int iPresses)
var
int iLoop
while iLoop <= iPresses
TypeKey (sKey)
Pause() ;The pause is essential
let iLoop = iLoop + 1
endwhile
EndFunction

Int Function ArePropertiesPainsOpen ()
var
int iLoop,
int iCount,
handle hWnd,
int Visible
let iLoop = 1
let iCount = 0
let hWnd = GetFirstChild( ghPropertiesWindow)
while iLoop <= 6
let Visible = IsWindowVisible (hWnd)
if iLoop == 1 && Visible then
let iCount = iCount+1
elif iLoop == 2 && Visible then
let iCount = iCount+1
elif iLoop == 3 && !Visible then
let iCount = iCount+1
elif iLoop == 4 && Visible then
let iCount = iCount+1
elif iLoop == 5 && !Visible then
let iCount = iCount+1
elif iLoop == 6 && Visible then
let iCount = iCount+1
endif
let hWnd = GetNextWindow( GetNextWindow( GetNextWindow( hWnd)))
let iLoop = iLoop + 1
 endwhile
if iCount != 6 then
return false
endif
return true
EndFunction





Int Function ConfigurePropertiesWindow (string sPainsToOpen)
var
string AllPains,
handle hWnd,
int Index,
int ShouldOpen,
int NumSuccesses
;get all the handles for all pains
let AllPains = GetPropertiesPainsHandles ()
;first close all pains which should be closed
;must be done this way round
let Index = 1
while Index < 7
let ShouldOpen = StringToInt( SubString( sPainsToOpen, Index, 1))
let hWnd = StringToHandle( StringSegment (AllPains, delimiter, Index))
if !ShouldOpen && OpenPropertiesPain (hWnd, false) then
let NumSuccesses = NumSuccesses + 1
endif
let Index = Index + 1
endwhile
;now open all pains which should be open
let Index = 1
while Index < 7
let ShouldOpen = StringToInt( SubString( sPainsToOpen, Index, 1))
let hWnd = StringToHandle( StringSegment (AllPains, delimiter, Index))
if ShouldOpen && OpenPropertiesPain (hWnd, true) then
let NumSuccesses = NumSuccesses + 1
endif
let Index = Index + 1
endwhile
PCCursor ()
if NumSuccesses == 6 then
return true
else
return false
endif
EndFunction

Int Function OpenPropertiesPain (handle hWnd, int Open)
var
int IsVisible,
handle hButton
let IsVisible = IsWindowVisible(hWnd)
if Open == IsVisible then
return TRUE
endif
JAWSCursor()
MoveToWindow (GetNextWindow(hWnd))
LeftMouseButton ()
Pause ()
let IsVisible = IsWindowVisible(hWnd)
if Open == IsVisible then
return TRUE
endif
return false
EndFunction

Void Function PropertiesPainsMode ()
let giFixingToolWindows = True
JawsCursor()
Say (MsgPropertiesNavigationOn, OT_LINE)
AddHook (HK_Script, "PropertiesPainsHook")
EndFunction

Int Function PropertiesPainsHook (string NameOfScript)
if NameOfScript == "SayNextCharacter" then
DoPropertiesNavigation (UnitMove_Next)
return False
elif NameOfScript == "SayPriorCharacter" then
DoPropertiesNavigation (UnitMove_Prior)
return False
elif NameOfScript == "SpaceBar" then
TogglePropertiesPain (giPropertiesPain)
return False
elif NameOfScript == "ScreenSensitiveHelp" then
SayFormattedMessage(OT_No_Disable, MsgPropertiesNavigationHelp)
return False
endif
let giFixingToolWindows = False
RestoreCursor()
RemoveHook (HK_Script, "PropertiesPainsHook")
Say (MsgPropertiesNavigationOff, OT_LINE)
SetFocus(GetScoreWindow())
return False
EndFunction

Void Function DoPropertiesNavigation (int Direction)
var
string AllPains,
handle hPain,
string msg
if direction == UnitMove_Next && giPropertiesPain < 6 then
let giPropertiesPain = giPropertiesPain + 1
elif direction == UnitMove_Prior && giPropertiesPain > 1 then
let giPropertiesPain = giPropertiesPain - 1
else
Beep()
endif
let AllPains = GetPropertiesPainsHandles ()
let hPain = StringToHandle( StringSegment (AllPains, delimiter, giPropertiesPain))
if IsWindowVisible (hPain) then
let msg = MsgOpen
else
let msg = MsgClose
endif
let msg = GetWindowName (GetNextWindow(hPain)) +space+ Msg
Say( msg, OT_NO_DISABLE)
EndFunction

Void Function TogglePropertiesPain (int iPain)
var
string AllPains,
handle hButton,
handle hPain,
string sPain,
string msg
let AllPains = GetPropertiesPainsHandles()
let hPain = StringToHandle( StringSegment (AllPains, delimiter, iPain))
let hButton = GetNextWindow (hPain)
MoveToWindow (hButton)
LeftMouseButton ()
Pause ()
let msg = StringSegment( gsPropertiesPains, delimiter, iPain)
if IsWindowVisible (hPain) then
let msg = msg +space+ MsgOpen
else
let msg = msg +space+ MsgClose
endif
Say( msg, OT_NO_DISABLE)
EndFunction


String Function GetPropertiesPainsHandles ()
var
handle hWnd,
string List
let hWnd = GetFirstChild (ghPropertiesWindow)
while hWnd
if GetWindowTypeCode (hWnd) == WT_Dialog then
let List = List + IntToString(hWnd) +delimiter
endif
let hWnd = GetNextWindow(hWnd)
endwhile
return StringChopRight (List, 1)
EndFunction


String Function ListOpenPropertiesPains ()
var
handle hWnd,
string List,
int NumPains,
int Index,
string FirstPain
let hWnd = GetFirstChild (ghPropertiesWindow)
while hWnd
if GetWindowTypeCode (hWnd) == WT_Dialog && IsWindowVisible (hWnd) then
let List = List + GetWindowName (GetNextWindow (hWnd)) +delimiter
endif
let hWnd = GetNextWindow(hWnd)
endwhile
;remove the last delimiter
let List = StringChopRight (List, 1)
let NumPains = StringSegmentCount (List, delimiter)
if NumPains == 1 then
SayString("hahaha")
return FormatString (MsgOpenPropertiesPainsSingle, List)
endif
let Index = 1
;Extract each name from the list except the last entry.
while Index < NumPains
let FirstPain = FirstPain + StringSegment(List, delimiter, Index) +comma+space
let Index = Index+1
endwhile
;strip the last space and comma away
let FirstPain = StringChopRight (FirstPain, 2)
return FormatString (MsgOpenPropertiesPainsPlural, FirstPain, StringSegment (List, delimiter, NumPains))
EndFunction


HANDLE Function GetSibeliusFrameWindow ()
var
handle hWnd
;some of the tool windows have their own main window so the Sibelius frame window needs to be found manually.
let hWnd = GetAppMainWindow (GetFocus())
;the Sibelius frame window is the last in the top level windows created by Sibelius
;so we can search each next window at this level to find it
while hWnd
if GetWindowClass(hWnd) == cWcMDIFrame &&
GetWindowName(hWnd) == Name_App then
return hWnd
endif
let hWnd = GetNextWindow (hWnd)
endwhile
EndFunction


HANDLE Function GetScoreWindow ()
var
handle hWnd,
handle hScore
;some of the tool windows have their own main window so the Sibelius frame window needs to be found manually.
let hWnd = GetSibeliusFrameWindow ()
if !hWnd then
SayFormattedMessage (OT_NO_DISABLE, MsgNoFrameWindow)
return
endif
let hScore = GetFirstChild( GetFirstChild (hWnd))
if GetWindowClass (hScore) == cWcMDIChild then
return hScore
endif
return False
EndFunction
